#include <stdio.h>	// printf es hasonlok...
#include <stdlib.h>	// malloc, free
#include <string.h>	// strcmp, strcpy

// Itt taroljuk az adatokat. Hogy konnyen ki lehessen menteni a dinamikus lista tartalmat,
// ezert a lista egyes elemei az adatokat egyetlen, ilyen "struct adatoktipus" tipusu mezoben
// (valtozoban) fogjak tarolni.
struct adatoktipus {
	char nev[40];
	int kor;
};

// Ez pedig a dinamikus list egy eleme.
struct elemtipus {
	struct adatoktipus adatok;	// Minden adat itt van. Ha ezt a valtozot kimentjuk,
								// minden fontos adatok kimentettunk.
	struct elemtipus *kov;		// Pointer a kovetkezo listaelemre.
};

/* ---------------------- Dinamikus lista muveletek -------------- */ 
// A jo oreg add fuggveny. Most az uj elembe toltendo adatokat egyetlen parameterrel
// kell megadni: ki kell tolteni egy "struct adatoktipus" tipusu valtozot,
// amit utana (egy ra mutato pointerrel) at kell adni az add-nek.
struct elemtipus *add(struct elemtipus *eleje, struct adatoktipus *ujadatok)
{
	// Memoriafoglalas
	struct elemtipus *ujelem = (struct elemtipus *)malloc(sizeof(struct elemtipus));
	if (!ujelem) {	// Sikerult?
		printf("Nem sikerult... :(\n");
		return eleje;	// Marad az eddigi eleje
	}
	// Ertekadasok az uj elemnek
	strcpy(ujelem->adatok.nev,ujadatok->nev);	// Masoljuk a kapott adatokot
												// Az ujelem->adatok mar nem pointer, igy utana pont (.) kell.
	ujelem->adatok.kor = ujadatok->kor;			// Itt is masolunk.
	// Dinamikus lista rendbe rakasa
	ujelem->kov = eleje;	// Befuzes, mint korabban.
	return ujelem;		// Most mar ez a lista uj eleme
}

void list(struct elemtipus *eleje)
{
	struct elemtipus *akt = NULL;
	for (akt=eleje; akt!=NULL; akt=akt->kov) {
		printf("Nev: %s, kor: %d\n",akt->adatok.nev,akt->adatok.kor);
			// Itt csak annyi valtozott, hogy az "akt->adatok"-on belulrol vesszuk az adatokat.
	}
}

// A clean fuggveny ugyanolyan, mint eddig.
struct elemtipus *clean(struct elemtipus *eleje)
{
	struct elemtipus *akt = eleje;
	struct elemtipus *kovetkezo = NULL;
	while (akt) {
		kovetkezo = akt->kov;
		free(akt);
		akt = kovetkezo;
	}
	return NULL;
}

/* ---------------------- Fajlmuveletek -------------- */ 

// Ha betoltunk egy fajlbol elemeket, akkor hasznalni fogjuk az "add" fuggvenyt,
// ezert valtozhat a lista eleje. Ezt visszateresi ertekkent adjuk vissza.
struct elemtipus *loadfromfile(struct elemtipus *eleje, char *filenev)
{
	struct adatoktipus ideiglenesadat;	// Ideiglenes tarolo valtozo
	int n;	// Itt taroljuk, hogy hany adatblokkot sikerult betolteni.

	FILE *F = fopen(filenev,"rb");	// Megnyitjuk a fajt olvasasra, binarisan.
	if (!F) {	// Sikerult?
		printf("A fajl megnyitasa nem sikerult...\n");
		return eleje;	// Ha nem, a lista eleje nem valtozott, mert nem kerult be uj elem.
	}
	printf("Fajlbol olvasas: ");
	// Az feof(F) akkor ad igaz erteket, ha az F fajl veget ert. Ezt akkor veszi eszre, amikor
	// egy olvasasi muvelet nem sikerult. Viszont vigyazni kell, mert ha egy fread nem sikerul,
	// akkor az utana leirt "add" meg lefutna, ha nem ellenoriznenk "n" erteket. Akkor pedig az utolso
	// beolvasott erteket ketszer venne fel!
	while (!feof(F)) {	// Amig nincs vege a fajlnak...
		// Beolvasunk egy "struct adatoktipus" meretu adatmennyiseget.
		// Egy ideiglenes taroloba ("ideiglenesadat") toltjuk be, ahonnan majd az "add" fuggveny
		// kimasolja az adatokat, amikor felvesszuk a dinamikus listaba.
		// A visszateresi ertek azt adja meg, hogy hany ilyen blokkot sikerult beolvasni. (1-et akarunk.)
		n = fread(&ideiglenesadat,sizeof(struct adatoktipus),1,F);
		if (n==1) 	// Ha sikerult is beolvasni az adatokat (n==0, ha nem sikerult), akkor
		{
			// A szokasos modon felvesszuk az uj adatokat a dinamikus listaba. Atadunk egy az
			// ideiglenes tarolo helyre mutato pointert, hogy onnan vegye az "add" az adatokat.
			eleje = add(eleje,&ideiglenesadat);	// eleje-nek uj erteket adhat!!!
			printf(".");
		}
	}
	fclose(F);	// Ha vege a fajlnak, akkor lezarjuk.
	printf("OK\n");
	return eleje;	// Visszaadjuk a lista uj elejet.
}

// Dinamikus lista tartalmanak kimentese fajlba.
void savetofile(struct elemtipus *eleje, char *filenev)
{
	struct elemtipus *akt = NULL;	// Itt taroljuk, hogy eppen hol tartunk
	FILE *F = fopen(filenev,"wb");	// Megnyitjuk a fajlt binarisan irasra.
	if (!F) {	// Sikerult?
		printf("A fajl megnyitasa nem sikerult...\n");
		return;
	}
	printf("Fajlba iras: ");
	// Vegigmegyunk az elemeken, pont ugy, mintha listazni akarnank...
	for (akt=eleje; akt!=NULL; akt=akt->kov) {
		// Amit kimentunk, az az aktualis elem ("akt" mutat ra) "adatok" mezoje.
		fwrite(&(akt->adatok),sizeof(struct adatoktipus),1,F);
		printf(".");
	}
	fflush(F);	// Fajlba iras utan cache kiuritese (minden keruljon ki a fajlba a lezaras elott).
	fclose(F);	// Fajl lezarasa
	printf("OK\n");
}

/* ---------------------- Foprogram -------------- */ 

// A program az elso inditaskor letrehozza a "lista.dat" fajlt, a masodiknal pedig mar
// onnan olvassa be az adatokt.
// Mivel az "add" fuggveny mindig a lista elejere fuzi az uj elemet, az 1-2-3 sorrendben felvett
// elemek 3-2-1 sorrendben kerulnek a listaba, majd a fajlba is. Innen viszont visszaolvasaskor
// (mivel ujra az "add" fuggveny veszi fel oket a listaba, ujra forditott (1-2-3) sorrendben, kerulnek
// a listaba, igy a masodik futtataskor mar igy jelennek meg a listazasnal.
// Igy gyorsabb az "add", mert nem kell megkeresnie a lista veget minden beszuraskor, de ha valakinek
// olyan program kell, ahol ez a sorrendfordulas zavaro, akkor az "add" fuggvenyt atalakitva ki lehet
// kuszobolni.
int main()
{
	struct elemtipus *start = NULL;	// A dinamikus lista eleje, mint eddig...
	struct adatoktipus adat;		// Nem dinamikusan foglaljuk le!

	// Megprobaljuk betolteni az adatokat egy fajlbol.
	start = loadfromfile(start,"lista.dat");
	// Ha nem sikerul, akkor a start meg mindig NULL, mert nem valtozott az erteke.
	// Ez azt jelenti, hogy meg nem valtozik a fajl, ezert most helyben felveszunk elemeket a meg mindig
	// ures dinamikus listaba, aztan pedig kimentjuk, hogy letrejojjon a file.
	if (start==NULL) {
		printf("--- Felveszek 3 elemet, mert meg nincs meg a file... ");
		strcpy(adat.nev,"Elso");	// A felvenni kivant ertekeket ideiglenesen berakjuk az "adat" valtozoba.
		adat.kor=1;
		start = add(start,&adat);	// A listaba felvetetunk egy elemet, az ertekeket az "adat"-bol masoltatjuk ki.
		strcpy(adat.nev,"Masodik");
		adat.kor=2;
		start = add(start,&adat);
		strcpy(adat.nev,"Harmadik");
		adat.kor=3;
		start = add(start,&adat);
		printf("OK\n");

		// Mentes fajlba. Elvileg most mar letezni fog, igy a kovetkezo inditaskor mar sikerul a betoltes.
		savetofile(start,"lista.dat");
	}
	else {	// Ha start!=NULL, akkor sikerult a betoltes.
		printf("Be tudtam olvasni a fajlt! Vigyazz! A sorrend ujra megfordult!\n");
	}

	// Igy vagy ugy, de most mar vannak elemek a dinamikus listaban.
	// Most kilistazzuk oket.
	printf("--- Listazas:\n");
	list(start);

	// Aztan a vegen torlunk mindent, ahogy illik.
	printf("--- Torlok mindent... ");
	start = clean(start);
	printf("OK\n");

	printf("--- Kesz ---\n");
	return 0;	// Minden OK, vege a programnak...
}
